﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.SqlClient;
using System.IO;
using System.Linq;
using System.Text.RegularExpressions;
using System.Threading.Tasks;
using System.Windows.Forms;
using Microsoft.Win32;

namespace ExinSearchRepeatedSmsTool
{
    public partial class MainForm : Form
    {
        private List<Template> _templates;
        public MainForm()
        {
            InitializeComponent();
        }

        private void MainForm_Load(object sender, EventArgs e)
        {
            BindTemplate();

        }

        private void BindTemplate()
        {
            _templates = LoadTemplate().ToList();
            _templates.Add(new Template
            {
                Index = -1,
                Description = "-------------请选择-------------"
            });
            _templates.Sort((x, y) => x.Index.CompareTo(y.Index));
            cbbTemplate.DataSource = _templates;
        }

        private void BindParameters(Template template)
        {
            gvParameters.Rows.Clear();
            txtConnString.Clear();
            if (!template.Enabled) return;
            txtConnString.Text = template.ConnectionString;

            template.Parameters.ToList().ForEach(p =>
            {
                var drow = new DataGridViewRow();
                var cell = new DataGridViewTextBoxCell { Value = p.PDes };
                drow.Cells.Add(cell);
                gvParameters.Rows.Add(drow);
                p.GetValue = () =>
                {
                    var val = drow.Cells["ColValue"].Value;
                    if (val == null) throw new InvalidOperationException(string.Format("还没有为参数 [{0}] 写入值", p.PDes));
                    return val.ToString();
                };
            });
        }


        private IEnumerable<Template> LoadTemplate()
        {
            var templatePath = AppDomain.CurrentDomain.BaseDirectory + "//template";
            var files = Directory.GetFiles(templatePath);
            foreach (var file in files)
            {
                var fs = File.OpenRead(file);
                var sr = new StreamReader(fs);
                var content = sr.ReadToEnd();
                sr.Close();
                fs.Close();

                var template = new Template(content);
                yield return template;
            }

        }

        private void cbbTemplate_SelectedIndexChanged(object sender, EventArgs e)
        {
            BindParameters(CurrentTemplate);
        }

        private Template CurrentTemplate
        {
            get { return cbbTemplate.SelectedItem as Template; }
        }

        private void ExceptionHanding(Action action)
        {
            try
            {
                action();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "出错了", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void btnExecute_Click(object sender, EventArgs e)
        {
            ExceptionHanding(() =>
            {
                var currentTemplate = CurrentTemplate;
                if (!currentTemplate.Enabled) throw new Exception("请检查模版文件，请确定是否准确配置了连接字符串和执行内容");
                var command = new SqlCommand
                {
                    CommandText = CurrentTemplate.RealSql,
                    CommandType = CommandType.Text,
                    Connection = new SqlConnection(CurrentTemplate.ConnectionString)
                };
                command.Connection.Open();
                var da = new SqlDataAdapter(command);
                var ds = new DataSet();
                btnExecute.Text = "正在执行....";
                btnExecute.Enabled = false;
                var task = new Task(() =>
                {
                    ExceptionHanding(() => da.Fill(ds));
                    Action action = null;
                    action = () =>
                    {
                        if (InvokeRequired)
                        {
                            Invoke(action);
                        }
                        else
                        {
                            ExceptionHanding(() =>
                            {
                                btnExecute.Text = "执行";
                                btnExecute.Enabled = true;
                                groupResultContainer.Controls.Clear();
                                AddControlForTables(ds);
                            });
                        }

                    };
                    action();
                });

                task.Start();
            });
        }

        private void AddControlForTables(DataSet ds)
        {
            Control parentControl = groupResultContainer;
            foreach (var table in ds.Tables)
            {
                var dv = new DataGridView
                {
                    Dock = DockStyle.Fill,
                    //Height = groupResultContainer.Height / ds.Tables.Count
                };
                var splitContainer = new SplitContainer
                {
                    Dock = DockStyle.Fill,
                    Orientation = Orientation.Horizontal
                };
                splitContainer.Panel1.Controls.Add(dv);
                parentControl.Controls.Add(splitContainer);
                dv.DataSource = table;
                dv.ReadOnly = true;
                parentControl = splitContainer.Panel2;
                dv.CellMouseDoubleClick += dataGridView_CellMouseDoubleClick;
            }
        }

        private void btnPreviewSql_Click(object sender, EventArgs e)
        {
            try
            {
                if (CurrentTemplate.Enabled)
                    new SqlForm(CurrentTemplate.RealSql).Show();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "出错了", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        readonly Regex _httpRegex = new Regex(@"(http|ftp|https):\/\/[\w\-_]+(\.[\w\-_]+)+([\w\-\.,@?^=%&amp;:/~\+#]*[\w\-\@?^=%&amp;/~\+#])?");
        private void dataGridView_CellMouseDoubleClick(object sender, DataGridViewCellMouseEventArgs e)
        {
            ExceptionHanding(() =>
            {
                if (e.RowIndex < 0 || e.ColumnIndex < 0) return;
                var dv = sender as DataGridView;
                if (dv != null)
                {
                    var value = dv.Rows[e.RowIndex].Cells[e.ColumnIndex].Value;
                    if (value == null) return;
                    var valueString = value.ToString();
                    if (_httpRegex.IsMatch(valueString))
                    {
                        var key = Registry.ClassesRoot.OpenSubKey(@"http\shell\open\command\");
                        if (key != null)
                        {
                            string s = key.GetValue("").ToString();
                            s = new Regex(@"""([^""]*)""").Match(s).Groups[1].Value;
                            System.Diagnostics.Process.Start(s, valueString);
                        }
                    }
                }
            });
        }


    }
}
