﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text.RegularExpressions;

namespace ExinSearchRepeatedSmsTool
{
    public class Template
    {
        private static readonly Regex ConnectionSringRegex = new Regex(@"\{connectionString:([^\}]*)\}");
        private static readonly Regex DescriptionRegex = new Regex(@"\{description:([^\}]*)\}");
        private static readonly Regex ParameterRegex = new Regex(@"\{(\w*),([^\}]*)\}");

        public Template(string content)
        {
            Content = content;
            ConnectionString = GetConnectionSringFromString();
            Parameters = GetParameters().ToList();
            Description = GetDescriptionFromString();
        }



        public Template()
        {
        }

        public string RealSql
        {
            get
            {
                string result = Content;
                Parameters.ToList().ForEach(p =>
                {
                    result = result.Replace("{" + p.Name + "," + p.PDes + "}", p.Value);
                });
                return result;
            }
        }

        public int Index { get; set; }

        public string ConnectionString { get; set; }
        public string Description { get; set; }
        public string Content { get; set; }

        public IList<Parameter> Parameters { get; set; }

        public bool Enabled
        {
            get { return !string.IsNullOrEmpty(Content) && !string.IsNullOrEmpty(ConnectionString); }
        }

        private string GetDescriptionFromString()
        {
            return DescriptionRegex.Match(Content).Groups[1].Value;
        }

        private string GetConnectionSringFromString()
        {
            return ConnectionSringRegex.Match(Content).Groups[1].Value;
        }

        private IEnumerable<Parameter> GetParameters()
        {
            var matches = ParameterRegex.Matches(Content);
            var parameters = from Match match in matches
                   select new Parameter(
                        match.Groups[1].Value,
                        match.Groups[2].Value
                       );
            return parameters.Distinct(new ParameterEqualCompare());
        }
    }

    public class ParameterEqualCompare : IEqualityComparer<Parameter>
    {
        public bool Equals(Parameter x, Parameter y)
        {
            return x.Name == y.Name;
        }

        public int GetHashCode(Parameter obj)
        {
            return obj.Name.GetHashCode();
        }
    }

    public class Parameter
    {
        public string Name { get; private set; }
        public Type CType { get; private set; }

        public string PDes { get; private set; }

        public Func<string> GetValue; 

        public Parameter(string name, string pdes)
        {
            Name = name;
            PDes = pdes;
           
        }
       
        public string Value
        {
            get { return GetValue(); }
        }
    }
}
